@@ -15,6 +15,7 @@ from typing import (Tuple, Callable, Optional, Union, Any, Dict, TypeVar,
 
 import numpy as np
 from scipy import interpolate, ndimage
+import matplotlib.pyplot as plt
 
 from ..grids.base import (GridBase, discretize_interval, DimensionError,
                           DomainError)
@@ -62,8 +63,8 @@ class FieldBase(metaclass=ABCMeta):
             label (str, optional):
                 Name of the field
         """
-        self.grid = grid
-        self._data: Any = data
+        self._grid = grid
+        self._data: np.ndarray = data
         self.label = label
         self._logger = logging.getLogger(self.__class__.__name__)
 
@@ -115,12 +116,12 @@ class FieldBase(metaclass=ABCMeta):
             if 'class' in fp.attrs:
                 # this should be a field collection
                 assert fp.attrs['class'] == 'FieldCollection'
-                obj = FieldCollection._from_dataset(fp)
+                obj = FieldCollection._from_hdf_dataset(fp)
                 
             elif len(fp) == 1:
                 # a single field is stored in the data
                 dataset = fp[list(fp.keys())[0]]  # retrieve only dataset
-                obj = cls._from_dataset(dataset)  # type: ignore
+                obj = cls._from_hdf_dataset(dataset)  # type: ignore
                 
             else:
                 raise RuntimeError('Multiple data fields were found in the '
@@ -129,7 +130,7 @@ class FieldBase(metaclass=ABCMeta):
                 
                 
     @classmethod
-    def _from_dataset(cls, dataset) -> "FieldBase":
+    def _from_hdf_dataset(cls, dataset) -> "FieldBase":
         """ construct a field by reading data from an hdf5 dataset """
         class_name = dataset.attrs['class']
         field_cls = cls._subclasses[class_name]
@@ -138,6 +139,12 @@ class FieldBase(metaclass=ABCMeta):
         label = dataset.attrs['label'] if 'label' in dataset.attrs else None
         return field_cls(grid, data=dataset, label=label)  # type: ignore
 
+    
+    @property
+    def grid(self) -> GridBase:
+        """ GridBase: The grid on which the field is defined """
+        return self._grid
+
 
     def to_file(self, filename: str, **kwargs):
         r""" store field in a file
@@ -148,8 +155,10 @@ class FieldBase(metaclass=ABCMeta):
         are supported. 
         
         Args:
-            filename (str): Path where the data is stored
-            \**kwargs: Additional parameters may be supported for some formats 
+            filename (str):
+                Path where the data is stored
+            \**kwargs:
+                Additional parameters may be supported for some formats 
         """
         extension = Path(filename).suffix.lower()
         
@@ -213,7 +222,7 @@ class FieldBase(metaclass=ABCMeta):
     
          
     @property
-    def data(self):
+    def data(self) -> np.ndarray:
         """ :class:`numpy.ndarray`: discretized data at the support points """
         return self._data
     
@@ -260,8 +269,10 @@ class FieldBase(metaclass=ABCMeta):
             
     
     def __eq__(self, other):
-        """ test for equality """
-        return (self.state == other.state and
+        """ test fields for equality, ignoring the label """
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+        return (self.grid == other.grid and
                 np.array_equal(self.data, other.data))
     
 
@@ -572,7 +583,6 @@ class FieldBase(metaclass=ABCMeta):
             
         # store the result to a file if requested
         if filename:
-            import matplotlib.pyplot as plt
             plt.savefig(filename)
             
         return res
@@ -876,8 +886,6 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
                 remaining arguments are are passed to
                 :func:`matplotlib.pyplot.imsave` to affect the appearance.
                         """
-        from matplotlib.pyplot import imsave
-
         # obtain image data
         get_image_args = {}
         for key in ['performance_goal', 'scalar']:
@@ -886,7 +894,7 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         img = self.get_image_data(**get_image_args)
         
         kwargs.setdefault('cmap', 'gray')
-        imsave(filename, img['data'], origin='lower', **kwargs)
+        plt.imsave(filename, img['data'], origin='lower', **kwargs)
     
 
     def _make_interpolator_scipy(self, method: str = 'linear',
@@ -1310,7 +1318,8 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             Gives the standard deviation of the smoothing in real length units
             (default: 1)
         out (FieldBase, optional):
-            Optional field into which the smoothed data is stored
+            Optional field into which the smoothed data is stored. Setting this
+            to the input field enables in-place smoothing.
         label (str, optional):
             Name of the returned field
 
@@ -1368,7 +1377,6 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         Returns:
             Instance of the line returned by `plt.plot`
         """
-        import matplotlib.pyplot as plt
         # obtain data
         line_data = self.get_line_data(extract=extract)
         
@@ -1425,8 +1433,6 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         Returns:
             Result of :func:`matplotlib.pyplot.imshow`
         """
-        import matplotlib.pyplot as plt
-        
         # obtain image data
         get_image_args = {}
         for key in ['performance_goal', 'scalar']:
@@ -1494,8 +1500,6 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             Result of :func:`matplotlib.pyplot.quiver` or
             :func:`matplotlib.pyplot.streamplot`
         """
-        import matplotlib.pyplot as plt
-        
         # obtain image data
         get_image_args = {}
         # FIXME: rename scalar_method to method
