@@ -55,13 +55,13 @@ class FieldCollection(FieldBase):
 
         # create the list of underlying fields        
         if copy_fields:
-            self.fields = [field.copy() for field in fields]
+            self._fields = [field.copy() for field in fields]
         else:
-            self.fields = fields  # type: ignore
+            self._fields = fields  # type: ignore
         
         # extract data from individual fields
         new_data: List[np.ndarray] = []
-        self.slices: List[slice] = []
+        self._slices: List[slice] = []
         dof = 0  # count local degrees of freedom
         for field in self.fields:
             if not isinstance(field, DataFieldBase):
@@ -71,7 +71,7 @@ class FieldCollection(FieldBase):
             start = len(new_data)
             this_data = field._data_flat
             new_data.extend(this_data)
-            self.slices.append(slice(start, len(new_data)))
+            self._slices.append(slice(start, len(new_data)))
             dof += len(this_data)
                 
         # combine into one data field
@@ -92,7 +92,7 @@ class FieldCollection(FieldBase):
         if not copy_fields:
             for i, field in enumerate(self.fields):
                 field_shape = field.data.shape
-                field._data_flat = self.data[self.slices[i]]
+                field._data_flat = self.data[self._slices[i]]
                 
                 # check whether the field data is based on our data field
                 assert field.data.base is self.data
@@ -159,6 +159,19 @@ class FieldCollection(FieldBase):
         else:
             raise TypeError(f'Unsupported index {index}')
 
+
+    @property
+    def fields(self) -> List[DataFieldBase]:
+        """ list: the fields of this collection """
+        return self._fields
+
+        
+    def __eq__(self, other):
+        """ test fields for equality, ignoring the label """
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+        return self.fields == other.fields
+    
         
     @classmethod
     def from_state(cls, state: Dict[str, Any], grid: GridBase,  # type: ignore
@@ -180,13 +193,13 @@ class FieldCollection(FieldBase):
 
 
     @classmethod
-    def _from_dataset(cls, dataset) -> "FieldCollection":
+    def _from_hdf_dataset(cls, dataset) -> "FieldCollection":
         """ construct the class by reading data from an hdf5 dataset """
         count = dataset.attrs['count']  # number of fields
         label = dataset.attrs['label'] if 'label' in dataset.attrs else None
         
         # reconstruct all fields
-        fields = [FieldBase._from_dataset(dataset[f"field_{i}"])
+        fields = [FieldBase._from_hdf_dataset(dataset[f"field_{i}"])
                   for i in range(count)]
              
         return cls(fields, label=label)  # type: ignore
