@@ -4,8 +4,7 @@ Defines a tensorial field of rank 2 over a grid
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 '''
 
-from typing import (Tuple, Optional, Union, Any, Dict, Callable,
-                    List, TypeVar, Iterator, TYPE_CHECKING)  # @UnusedImport
+from typing import Tuple, Optional, Union, Any, Dict, Callable, TYPE_CHECKING
 
 import numpy as np
 import numba as nb
@@ -15,6 +14,8 @@ from .scalar import ScalarField
 from .vectorial import VectorField
 
 from ..tools.numba import jit
+from ..tools.docstrings import fill_in_docstring
+
 
 
 if TYPE_CHECKING:
@@ -158,17 +159,19 @@ class Tensor2Field(DataFieldBase):
         return dot
     
           
+    @fill_in_docstring
     def divergence(self, bc: "BoundariesData",
                    out: Optional[VectorField] = None,
                    label: str = 'divergence') -> VectorField:
         """ apply (tensor) divergence and return result as a field 
         
         Args:
-            bc: Gives the boundary conditions applied to fields that are
-                required for calculating the divergence.
-            out (VectorField, optional): Optional scalar field to which the 
-                result is written.
-            label (str, optional): Name of the returned field
+            bc: 
+                {ARG_BOUNDARIES}
+            out (VectorField, optional):
+                Optional scalar field to which the  result is written.
+            label (str, optional):
+                Name of the returned field
             
         Returns:
             VectorField: the result of applying the operator 
@@ -186,10 +189,7 @@ class Tensor2Field(DataFieldBase):
     @property
     def integral(self):
         """ :class:`numpy.ndarray`: integral of each component over space """
-        out = np.empty((self.grid.dim, self.grid.dim))
-        for i, j in np.ndindex(out.shape):
-            out[i, j] = self.grid.integrate(self.data[i, j])
-        return out
+        return self.grid.integrate(self.data)
     
     
     def transpose(self, label: str = 'transpose') -> 'Tensor2Field':
