@@ -4,7 +4,8 @@ Defines a vectorial field over a grid
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 '''
 
-from typing import Callable, Optional, Union, Any, Dict, List, TYPE_CHECKING
+from typing import (Callable, Optional, Union, Any, Dict, List, Sequence,
+                    TYPE_CHECKING)
 
 import numpy as np
 import numba as nb
@@ -12,7 +13,10 @@ import numba as nb
 
 from .base import DataFieldBase
 from .scalar import ScalarField
+from ..grids.base import GridBase, DimensionError
 from ..tools.numba import jit
+from ..tools.expressions import ScalarExpression
+from ..tools.docstrings import fill_in_docstring
 
   
 if TYPE_CHECKING:
@@ -53,8 +57,8 @@ class VectorField(DataFieldBase):
         grid = fields[0].grid
         
         if grid.dim != len(fields):
-            raise ValueError('Grid dimension and number of scalar fields '
-                             f'differ ({grid.dim} != {len(fields)})')
+            raise DimensionError('Grid dimension and number of scalar fields '
+                                 f'differ ({grid.dim} != {len(fields)})')
             
         data = []
         for field in fields:
@@ -64,6 +68,41 @@ class VectorField(DataFieldBase):
         return cls(grid, data, label)
     
 
+    @classmethod
+    def from_expression(cls, grid: GridBase, expressions: Sequence[str],
+                        label: str = None) -> "VectorField":
+        """ create a vector field on a grid from given expressions
+        
+        Args:
+            grid (:class:`~pde.grids.GridBase`):
+                Grid defining the space on which this field is defined
+            expressions (list of str):
+                A list of mathematical expression, one for each component of the
+                vector field. The expressions determine the values as a function
+                of the position on the grid. The expressions may contain
+                standard mathematical functions and they may depend on the axes
+                labels of the grid.
+            label (str, optional):
+                Name of the field
+        """
+        if isinstance(expressions, str) or len(expressions) != grid.dim:
+            axes_names = grid.axes + grid.axes_symmetric
+            raise ValueError(f'Expected {grid.dim} expressions for the '
+                             f'coordinates {axes_names}.') 
+        
+        # obtain the values at the grid points
+        points = {name: grid.cell_coords[..., i]
+                  for i, name in enumerate(grid.axes)}
+        
+        # evaluate the different components
+        data = []
+        for expression in expressions:
+            expr = ScalarExpression(expression=expression, signature=grid.axes) 
+            data.append(expr(**points))
+
+        return cls(grid=grid, data=data, label=label)
+        
+
     def __getitem__(self, key: int) -> ScalarField:
         """ extract a component of the VectorField """
         if not isinstance(key, int):
@@ -203,17 +242,19 @@ class VectorField(DataFieldBase):
         return out
         
 
+    @fill_in_docstring
     def divergence(self, bc: "BoundariesData",
                    out: Optional[ScalarField] = None,
                    label: str = 'divergence') -> ScalarField:
         """ apply divergence operator and return result as a field 
         
         Args:
-            bc: Gives the boundary conditions applied to fields that are
-                required for calculating the divergence.
-            out (ScalarField, optional): Optional scalar field to which the 
-                result is written.
-            label (str, optional): Name of the returned field
+            bc: 
+                {ARG_BOUNDARIES}
+            out (ScalarField, optional):
+                Optional scalar field to which the  result is written.
+            label (str, optional):
+                Name of the returned field
             
         Returns:
             ScalarField: the result of applying the operator 
@@ -228,17 +269,19 @@ class VectorField(DataFieldBase):
         return out
     
     
+    @fill_in_docstring
     def gradient(self, bc: "BoundariesData",
                  out: Optional['Tensor2Field'] = None,
                  label: str = 'gradient') -> 'Tensor2Field':
         """ apply (vecotr) gradient operator and return result as a field 
         
         Args:
-            bc: Gives the boundary conditions applied to fields that are
-                required for calculating the gradient.
-            out (Tensor2Field, optional): Optional scalar field to which the 
-                result is written.
-            label (str, optional): Name of the returned field
+            bc:
+                {ARG_BOUNDARIES}
+            out (Tensor2Field, optional):
+                Optional tensorial field to which the  result is written.
+            label (str, optional):
+                Name of the returned field
             
         Returns:
             Tensor2Field: the result of applying the operator 
@@ -255,17 +298,19 @@ class VectorField(DataFieldBase):
         return out
 
         
+    @fill_in_docstring
     def laplace(self, bc: "BoundariesData",
                 out: Optional['VectorField'] = None,
                 label: str = 'vector laplacian') -> 'VectorField':
         """ apply vector Laplace operator and return result as a field 
         
         Args:
-            bc: Gives the boundary conditions applied to fields that are
-                required for calculating the vector Laplacian.
-            out (VectorField, optional): Optional vector field to which the 
-                result is written.
-            label (str, optional): Name of the returned field
+            bc: 
+                {ARG_BOUNDARIES}
+            out (VectorField, optional):
+                Optional vector field to which the  result is written.
+            label (str, optional):
+                Name of the returned field
             
         Returns:
             VectorField: the result of applying the operator 
@@ -279,7 +324,7 @@ class VectorField(DataFieldBase):
     @property
     def integral(self) -> np.ndarray:
         """ :class:`numpy.ndarray`: integral of each component over space """
-        return np.array([self.grid.integrate(d) for d in self.data])
+        return self.grid.integrate(self.data)
                 
     
     def to_scalar(self, scalar: Union[str, int] = 'norm',
