@@ -16,18 +16,159 @@ This module implements differential operators on Cartesian grids
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>   
 '''
 
-from typing import Callable
+from typing import Callable, Tuple, Any
 
 import numba as nb
 import numpy as np
-from scipy import ndimage
+from scipy import ndimage, sparse
 
 from . import PARALLELIZATION_THRESHOLD_2D, PARALLELIZATION_THRESHOLD_3D
+from .common import make_laplace_from_matrix, make_poisson_solver
 from ..boundaries import Boundaries
 from ...tools.numba import jit_allocate_out
+from ...tools.docstrings import fill_in_docstring
 
 
 
+@fill_in_docstring
+def _get_laplace_matrix_1d(bcs) -> Tuple[Any, Any]:
+    """ get sparse matrix for laplace operator on a 1d Cartesian grid
+    
+    Args:
+        bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
+            {ARG_BOUNDARIES_INSTANCE}
+        
+    Returns:
+        tuple: A sparse matrix and a sparse vector that can be used to evaluate
+        the discretized laplacian
+    """    
+    dim_x = bcs.grid.shape[0]
+    matrix = sparse.dok_matrix((dim_x, dim_x))
+    vector = sparse.dok_matrix((dim_x, 1))
+    
+    for i in range(dim_x):
+        matrix[i, i] += -2
+        
+        if i == 0:
+            const, entries = bcs[0].get_data((-1,))
+            vector[i] += const
+            for k, v in entries.items():
+                matrix[i, k] += v
+        else:
+            matrix[i, i - 1] += 1
+            
+        if i == dim_x - 1:
+            const, entries = bcs[0].get_data((dim_x,))
+            vector[i] += const
+            for k, v in entries.items():
+                matrix[i, k] += v
+        else:
+            matrix[i, i + 1] += 1
+
+    matrix *= bcs.grid.discretization[0]**-2
+    vector *= bcs.grid.discretization[0]**-2
+            
+    return matrix, vector
+
+    
+
+@fill_in_docstring
+def _get_laplace_matrix_2d(bcs) -> Tuple[Any, Any]:
+    """ get sparse matrix for laplace operator on a 2d Cartesian grid
+    
+    Args:
+        bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
+            {ARG_BOUNDARIES_INSTANCE}
+        
+    Returns:
+        tuple: A sparse matrix and a sparse vector that can be used to evaluate
+        the discretized laplacian
+    """        
+    dim_x, dim_y = bcs.grid.shape
+    matrix = sparse.dok_matrix((dim_x * dim_y, dim_x * dim_y))
+    vector = sparse.dok_matrix((dim_x * dim_y, 1))
+    
+    bc_x, bc_y = bcs
+    scale_x, scale_y = bcs.grid.discretization**-2
+    
+    def i(x, y):
+        """ helper function for flattening the index
+        
+        This is equivalent to np.ravel_multi_index((x, y), (dim_x, dim_y))
+        """
+        return x * dim_y + y
+        
+    # set diagonal elements, i.e., the central value in the kernel
+    matrix.setdiag(-2 * (scale_x + scale_y))
+        
+    for x in range(dim_x):
+        for y in range(dim_y):
+            # handle x-direction
+            if x == 0:
+                const, entries = bc_x.get_data((-1, y))
+                vector[i(x, y)] += const * scale_x
+                for k, v in entries.items():
+                    matrix[i(x, y), i(k, y)] += v * scale_x
+            else:
+                matrix[i(x, y), i(x - 1, y)] += scale_x
+
+            if x == dim_x - 1:
+                const, entries = bc_x.get_data((dim_x, y))
+                vector[i(x, y)] += const * scale_x
+                for k, v in entries.items():
+                    matrix[i(x, y), i(k, y)] += v * scale_x
+            else:
+                matrix[i(x, y), i(x + 1, y)] += scale_x  
+                
+            # handle y-direction
+            if y == 0:
+                const, entries = bc_y.get_data((x, -1))
+                vector[i(x, y)] += const * scale_y
+                for k, v in entries.items():
+                    matrix[i(x, y), i(x, k)] += v * scale_y
+            else:
+                matrix[i(x, y), i(x, y - 1)] += scale_y
+
+            if y == dim_y - 1:
+                const, entries = bc_y.get_data((x, dim_y))
+                vector[i(x, y)] += const * scale_y
+                for k, v in entries.items():
+                    matrix[i(x, y), i(x, k)] += v * scale_y
+            else:
+                matrix[i(x, y), i(x, y + 1)] += scale_y
+
+    return matrix, vector
+
+
+
+@fill_in_docstring
+def _get_laplace_matrix(bcs: Boundaries):
+    """ get sparse matrix for laplace operator on a 1d Cartesian grid
+    
+    Args:
+        bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
+            {ARG_BOUNDARIES_INSTANCE}
+        
+    Returns:
+        tuple: A sparse matrix and a sparse vector that can be used to evaluate
+        the discretized laplacian
+    """    
+    dim = bcs.grid.dim
+    bcs.check_value_rank(0)
+    
+    if dim == 1:
+        result = _get_laplace_matrix_1d(bcs)
+    elif dim == 2:
+        result = _get_laplace_matrix_2d(bcs)
+    else:
+        raise NotImplementedError('Numba laplace operator not implemented '
+                                  f'for {dim:d} dimensions')
+                                      
+    return result
+
+
+
+@fill_in_docstring
 def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a laplace operator using the scipy module
     
@@ -35,7 +176,7 @@ def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -70,12 +211,13 @@ def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
 
         
     
+@fill_in_docstring
 def _make_laplace_numba_1d(bcs: Boundaries) -> Callable:
     """ make a 1d laplace operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -97,12 +239,13 @@ def _make_laplace_numba_1d(bcs: Boundaries) -> Callable:
 
     
     
+@fill_in_docstring
 def _make_laplace_numba_2d(bcs: Boundaries) -> Callable:
     """ make a 2d laplace operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -156,12 +299,13 @@ def _make_laplace_numba_2d(bcs: Boundaries) -> Callable:
     
 
 
+@fill_in_docstring
 def _make_laplace_numba_3d(bcs: Boundaries) -> Callable:
     """ make a 3d laplace operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -198,12 +342,13 @@ def _make_laplace_numba_3d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_laplace(bcs: Boundaries, method: str = 'auto') -> Callable:
     """ make a laplace operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the laplace operator.
             If method='auto', a suitable method is chosen automatically.
         
@@ -230,6 +375,9 @@ def make_laplace(bcs: Boundaries, method: str = 'auto') -> Callable:
         else:
             raise NotImplementedError('Numba laplace operator not implemented '
                                       f'for {dim:d} dimensions')
+                          
+    elif method == 'matrix':
+        laplace = make_laplace_from_matrix(*_get_laplace_matrix(bcs))
                                       
     elif method == 'scipy':
         laplace = _make_laplace_scipy_nd(bcs)
@@ -241,12 +389,13 @@ def make_laplace(bcs: Boundaries, method: str = 'auto') -> Callable:
 
 
 
+@fill_in_docstring
 def _make_gradient_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a gradient operator using the scipy module
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -268,12 +417,13 @@ def _make_gradient_scipy_nd(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_gradient_numba_1d(bcs: Boundaries) -> Callable:
     """ make a 1d gradient operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:    
         A function that can be applied to an array of values
@@ -295,12 +445,13 @@ def _make_gradient_numba_1d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_gradient_numba_2d(bcs: Boundaries) -> Callable:
     """ make a 2d gradient operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -331,12 +482,13 @@ def _make_gradient_numba_2d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_gradient_numba_3d(bcs: Boundaries) -> Callable:
     """ make a 3d gradient operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -371,12 +523,13 @@ def _make_gradient_numba_3d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_gradient(bcs: Boundaries, method: str = 'auto') -> Callable:
     """ make a gradient operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the gradient operator.
             If method='auto', a suitable method is chosen automatically.
         
@@ -414,6 +567,7 @@ def make_gradient(bcs: Boundaries, method: str = 'auto') -> Callable:
 
 
 
+@fill_in_docstring
 def _make_divergence_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a divergence operator using the scipy module
     
@@ -421,7 +575,7 @@ def _make_divergence_scipy_nd(bcs: Boundaries) -> Callable:
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -447,13 +601,14 @@ def _make_divergence_scipy_nd(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_divergence_numba_1d(bcs: Boundaries) -> Callable:
     """ make a 1d divergence operator using numba compilation
     
     Args:
         dim (int): The number of support points for each axes
         boundaries (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         dx (float): The discretization
         
     Returns:    
@@ -476,12 +631,13 @@ def _make_divergence_numba_1d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_divergence_numba_2d(bcs: Boundaries) -> Callable:
     """ make a 2d divergence operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -513,12 +669,13 @@ def _make_divergence_numba_2d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_divergence_numba_3d(bcs: Boundaries) -> Callable:
     """ make a 3d divergence operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -554,12 +711,13 @@ def _make_divergence_numba_3d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_divergence(bcs: Boundaries, method: str = 'auto') -> Callable:
     """ make a divergence operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the divergence operator.
             If method='auto', a suitable method is chosen automatically.
         
@@ -597,6 +755,7 @@ def make_divergence(bcs: Boundaries, method: str = 'auto') -> Callable:
 
 
 
+@fill_in_docstring
 def _make_vector_gradient_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a vector gradient operator using the scipy module
     
@@ -604,7 +763,7 @@ def _make_vector_gradient_scipy_nd(bcs: Boundaries) -> Callable:
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -628,12 +787,13 @@ def _make_vector_gradient_scipy_nd(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_vector_gradient_numba_1d(bcs: Boundaries) -> Callable:
     """ make a 1d vector gradient operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:    
         A function that can be applied to an array of values
@@ -650,12 +810,13 @@ def _make_vector_gradient_numba_1d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_vector_gradient_numba_2d(bcs: Boundaries) -> Callable:
     """ make a 2d vector gradient operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -674,13 +835,14 @@ def _make_vector_gradient_numba_2d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_vector_gradient_numba_3d(bcs: Boundaries) -> Callable:
     """ make a 3d vector gradient operator using numba compilation
     
     Args:
         shape (tuple): The number of support points for each axes
         boundaries (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         dx (float or list): The discretizations
         
     Returns:
@@ -702,12 +864,13 @@ def _make_vector_gradient_numba_3d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_vector_gradient(bcs: Boundaries, method: str = 'auto') -> Callable:
     """ make a vector gradient operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the vector gradient operator.
             If method='auto', a suitable method is chosen automatically
         
@@ -743,6 +906,7 @@ def make_vector_gradient(bcs: Boundaries, method: str = 'auto') -> Callable:
     return gradient
 
 
+@fill_in_docstring
 def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a vector Laplacian using the scipy module
     
@@ -750,7 +914,7 @@ def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -772,12 +936,13 @@ def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_vector_laplace_numba_1d(bcs: Boundaries) -> Callable:
     """ make a 1d vector Laplacian using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:    
         A function that can be applied to an array of values
@@ -794,12 +959,13 @@ def _make_vector_laplace_numba_1d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_vector_laplace_numba_2d(bcs: Boundaries) -> Callable:
     """ make a 2d vector Laplacian using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -818,12 +984,13 @@ def _make_vector_laplace_numba_2d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_vector_laplace_numba_3d(bcs: Boundaries) -> Callable:
     """ make a 3d vector Laplacian using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -844,12 +1011,13 @@ def _make_vector_laplace_numba_3d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_vector_laplace(bcs: Boundaries, method: str = 'auto') -> Callable:
     """ make a vector Laplacian on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the vector laplace operator.
             If method='auto', a suitable method is chosen automatically.
         
@@ -886,6 +1054,7 @@ def make_vector_laplace(bcs: Boundaries, method: str = 'auto') -> Callable:
 
 
 
+@fill_in_docstring
 def _make_tensor_divergence_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a tensor divergence operator using the scipy module
     
@@ -893,7 +1062,7 @@ def _make_tensor_divergence_scipy_nd(bcs: Boundaries) -> Callable:
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -921,12 +1090,13 @@ def _make_tensor_divergence_scipy_nd(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_tensor_divergence_numba_1d(bcs: Boundaries) -> Callable:
     """ make a 1d tensor divergence  operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:    
         A function that can be applied to an array of values
@@ -943,12 +1113,13 @@ def _make_tensor_divergence_numba_1d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_tensor_divergence_numba_2d(bcs: Boundaries) -> Callable:
     """ make a 2d tensor divergence  operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -967,12 +1138,13 @@ def _make_tensor_divergence_numba_2d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def _make_tensor_divergence_numba_3d(bcs: Boundaries) -> Callable:
     """ make a 3d tensor divergence  operator using numba compilation
     
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values    
@@ -993,12 +1165,13 @@ def _make_tensor_divergence_numba_3d(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_tensor_divergence(bcs: Boundaries, method: str = 'auto') -> Callable:
     """ make a tensor divergence operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the tensor divergence
             operator. If method='auto', a suitable method is chosen
             automatically.
@@ -1036,113 +1209,7 @@ def make_tensor_divergence(bcs: Boundaries, method: str = 'auto') -> Callable:
 
 
 
-# def rfftnfreq(shape: Tuple[int, ...], dx=1):
-#     """ Return the Discrete Fourier Transform sample frequencies (for usage
-#     with rfftn, irfftn).
-#     
-#     Args:
-#         shape (tuple): the length of each axis
-#         dx (float or sequence): the discretization along each axis. If a
-#             single number is given, the same discretization is assumed along 
-#             each axis
-#             
-#     Returns:
-#         numpy.ndarray
-#     """
-#     dim = len(shape)
-#     dx = np.broadcast_to(dx, (dim,))
-#     
-#     k2s = 0
-#     for i in range(dim):
-#         # get wave vector for axis i
-#         if i == dim - 1:
-#             k = np.fft.rfftfreq(shape[i], dx[i])
-#         else:
-#             k = np.fft.fftfreq(shape[i], dx[i])
-#         # add the squared components to all present ones
-#         k2s = np.add.outer(k2s, k**2)
-#         
-#     # get the magnitude at each position
-#     return np.sqrt(k2s)
-# 
-# 
-# 
-# def _make_poisson_solver_scipy_nd(shape: Tuple[int, ...], dx) -> Callable:
-#     """ make an operator that solves Poisson's equation on a Cartesian grid
-#     using scipy.
-#     
-#     Args:
-#         shape (tuple): The number of support points for each axes
-#         dx (float or list): The discretizations
-#         
-#     Returns:
-#         A function that can be applied to an array of values
-#     """        
-#     # prepare wave vector
-#     k2s = rfftnfreq(shape, dx)**2
-#         
-#     # TODO: accelerate the FFT using the pyfftw package
-#         
-#     def poisson_solver(arr):
-#         """ apply poisson solver to array `arr` """
-#         # forward transform
-#         arr = np.fft.rfftn(arr)
-# 
-#         # divide by squared wave vector
-#         arr.flat[0] = 0  # remove zero mode
-#         arr.flat[1:] /= k2s.flat[1:]
-# 
-#         # backwards transform
-#         arr = np.fft.irfftn(arr, shape)
-#         return arr
-#     
-#     return poisson_solver
-# 
-# 
-# 
-# def make_poisson_solver(shape: Tuple[int, ...], boundaries: Boundaries, dx=1,
-#                         method: str='auto') -> Callable:
-#     r""" make an operator that solves Poisson's equation on a Cartesian grid
-#     
-#     Denoting the current field by :math:`x`, we thus solve for :math:`y`,
-#     defined by the equation 
-# 
-#     .. math::
-#         \nabla^2 y(\boldsymbol r) = -x(\boldsymbol r)
-#      
-#         
-#     Currently, this is only supported for periodic boundary conditions.
-#     
-#     Args:
-#         shape (tuple): The number of support points for each axes
-#         boundaries (:class:`~pde.grids.boundaries.axes.Boundaries`):
-#             The boundary conditions. If the boundary conditions are not given
-#             they are assumed to be all periodic.
-#         dx (float or list): The discretizations
-#         
-#     Returns:
-#         A function that can be applied to an array of values
-#     """    
-#     dim = len(shape)
-#     if boundaries is not None:
-#         boundaries.check_dimensions(dim=dim)
-#         if not np.all(boundaries.periodic):
-#             raise NotImplementedError("Solving Poisson's equation is only "
-#                                       "implemented for periodic boundary "
-#                                       "conditions")
-# 
-#     if method == 'auto':
-#         method = 'scipy'
-# 
-#     if method == 'scipy':
-#         poisson_solver = _make_poisson_solver_scipy_nd(shape, dx)
-#     else:
-#         raise ValueError(f'Method `{method}` is not defined')
-#         
-#     return poisson_solver
-
-
-
+@fill_in_docstring
 def make_operator(op: str, bcs: Boundaries, method: str = 'auto') -> Callable:
     """ return a discretized operator for a Cartesian grid
     
@@ -1150,7 +1217,7 @@ def make_operator(op: str, bcs: Boundaries, method: str = 'auto') -> Callable:
         op (str): Identifier for the operator. Some examples are 'laplace',
             'gradient', or 'divergence'.
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         method (str): The method for implementing the operator
         
     Returns:
@@ -1171,14 +1238,16 @@ def make_operator(op: str, bcs: Boundaries, method: str = 'auto') -> Callable:
         return make_vector_laplace(bcs, method)
     elif op == 'tensor_divergence':
         return make_tensor_divergence(bcs, method)
-#     elif op == 'poisson_solver':
-#         return make_poisson_solver(shape, bc, dx, method)
+    elif op == 'poisson_solver' or op == 'solve_poisson' or op == 'poisson':
+        matrix, vector = _get_laplace_matrix(bcs)
+        return make_poisson_solver(matrix, vector, method)
     else:
         raise NotImplementedError(f'Operator `{op}` is not defined for '
                                   'Cartesian grids')
-    
+
 
 
 __all__ = ["make_laplace", "make_gradient", "make_divergence",
            "make_vector_gradient", "make_vector_laplace",
-           "make_tensor_divergence", "make_operator"]
+           "make_tensor_divergence", "make_poisson_solver", "make_operator"]
+    
\ No newline at end of file
