@@ -13,36 +13,29 @@ This module implements differential operators on polar grids
    
    
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
-
-
-.. The following contains text parts that are used multiple times below:
-
-.. |Description_polar| replace:: 
-  This function assumes polar symmetry of the grid, so that fields only
-  depend on the radial coordinate `r`. The radial discretization is defined as
-  :math:`r_i = r_\mathrm{min} + (i + \frac12) \Delta r` for
-  :math:`i=0, \ldots, N_r-1`,  where :math:`r_\mathrm{min}` is the radius of
-  the inner boundary, which is zero by default. Note that the radius of the 
-  outer boundary is given by
-  :math:`r_\mathrm{max} = r_\mathrm{min} + N_r \Delta r`.
 """
 
 from typing import Callable
 
+from scipy import sparse
+
+from .common import make_poisson_solver
 from .. import PolarGrid
 from ..boundaries import Boundaries
 from ...tools.numba import jit_allocate_out
+from ...tools.docstrings import fill_in_docstring
 
 
 
+@fill_in_docstring
 def make_laplace(bcs: Boundaries) -> Callable:
     """ make a discretized laplace operator for a polar grid
     
-    |Description_polar|
+    {DESCR_POLAR_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -87,14 +80,15 @@ def make_laplace(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_gradient(bcs: Boundaries) -> Callable:
     """ make a discretized gradient operator for a polar grid
     
-    |Description_polar|
+    {DESCR_POLAR_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -140,14 +134,15 @@ def make_gradient(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_divergence(bcs: Boundaries) -> Callable:
     """ make a discretized divergence operator for a polar grid
     
-    |Description_polar|
+    {DESCR_POLAR_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -211,14 +206,15 @@ def make_divergence(bcs: Boundaries) -> Callable:
 
     
     
+@fill_in_docstring
 def make_vector_gradient(bcs: Boundaries) -> Callable:
     """ make a discretized vector gradient operator for a polar grid
     
-    |Description_polar|
+    {DESCR_POLAR_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -240,14 +236,15 @@ def make_vector_gradient(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_tensor_divergence(bcs: Boundaries) -> Callable:
     """ make a discretized tensor divergence operator for a polar grid
     
-    |Description_polar|
+    {DESCR_POLAR_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -269,16 +266,74 @@ def make_tensor_divergence(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
+def _get_laplace_matrix(bcs):
+    """ get sparse matrix for laplace operator on a polar grid
+    
+    Args:
+        bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
+            {ARG_BOUNDARIES_INSTANCE}
+        
+    Returns:
+        tuple: A sparse matrix and a sparse vector that can be used to evaluate
+        the discretized laplacian
+    """    
+    assert isinstance(bcs.grid, PolarGrid)
+    bcs.check_value_rank(0)
+    
+    # calculate preliminary quantities
+    dim_r = bcs.grid.shape[0]
+    dr = bcs.grid.discretization[0]
+    rs = bcs.grid.axes_coords[0]
+    r_min, _ = bcs.grid.axes_bounds[0]
+    scale = 1 / dr**2
+
+    matrix = sparse.dok_matrix((dim_r, dim_r))
+    vector = sparse.dok_matrix((dim_r, 1))
+    
+    for i in range(dim_r):
+        matrix[i, i] += -2 * scale
+        scale_i = 1 / (2 * rs[i] * dr)
+        
+        if i == 0:
+            if r_min == 0:
+                matrix[i, i + 1] = 2 * scale
+                continue  # the special case of the inner boundary is handled
+            else:
+                const, entries = bcs[0].get_data((-1,))
+                factor = scale - scale_i
+                vector[i] += const * factor
+                for k, v in entries.items():
+                    matrix[i, k] += v * factor
+
+        else:
+            matrix[i, i - 1] = scale - scale_i
+            
+        if i == dim_r - 1:
+            const, entries = bcs[0].get_data((dim_r,))
+            factor = scale + scale_i
+            vector[i] += const * factor
+            for k, v in entries.items():
+                matrix[i, k] += v * factor
+                
+        else:
+            matrix[i, i + 1] = scale + scale_i
+            
+    return matrix, vector
+
+
+
+@fill_in_docstring
 def make_operator(op: str, bcs: Boundaries) -> Callable:
     """ make a discretized operator for a polar grid
     
-    |Description_polar|
+    {DESCR_POLAR_GRID}
 
     Args:
         op (str): Identifier for the operator. Some examples are 'laplace',
             'gradient', or 'divergence'.
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that takes the discretized data as an input and returns
@@ -296,6 +351,8 @@ def make_operator(op: str, bcs: Boundaries) -> Callable:
         return make_vector_gradient(bcs)
     elif op == 'tensor_divergence':
         return make_tensor_divergence(bcs)
+    elif op == 'poisson_solver' or op == 'solve_poisson' or op == 'poisson':
+        return make_poisson_solver(*_get_laplace_matrix(bcs))
     else:
         raise NotImplementedError(f'Operator `{op}` is not defined for '
                                   'polar grids')
