@@ -22,8 +22,10 @@ The trackers defined in this module are:
 from datetime import timedelta
 import inspect
 import sys
+import os.path
 import time
-from typing import Callable, Optional, Union, IO, List, Any  # @UnusedImport
+from typing import (Callable, Optional, Union, IO, List, Any,  # @UnusedImport
+                    TYPE_CHECKING)
 
 import numpy as np
 
@@ -38,6 +40,11 @@ from ..tools.docstrings import fill_in_docstring
 
 
 
+if TYPE_CHECKING:
+    import pandas  # @UnusedImport
+
+
+
 class CallbackTracker(TrackerBase):
     """ Tracker that calls a function periodically """
     
@@ -70,7 +77,8 @@ class CallbackTracker(TrackerBase):
         Args:
             field (:class:`~pde.fields.FieldBase`):
                 The current state of the simulation
-            t (float): The associated time
+            t (float):
+                The associated time
         """
         if self._num_args == 1:
             self._callback(field)
@@ -141,7 +149,8 @@ class ProgressTracker(TrackerBase):
         Args:
             field (:class:`~pde.fields.FieldBase`):
                 The current state of the simulation
-            t (float): The associated time
+            t (float):
+                The associated time
         """
         # show an update
         if self.progress_bar.total:
@@ -219,7 +228,8 @@ class PrintTracker(TrackerBase):
         Args:
             field (:class:`~pde.fields.FieldBase`):
                 The current state of the simulation
-            t (float): The associated time
+            t (float):
+                The associated time
         """
         data = f"c={field.data.mean():.3g}±{field.data.std():.3g}"
             
@@ -305,7 +315,8 @@ class PlotTracker(TrackerBase):
         Args:
             field (:class:`~pde.fields.FieldBase`):
                 The current state of the simulation
-            t (float): The associated time
+            t (float):
+                The associated time
         """
         self.plot.show_data(field, title=f'Time {t:g}')
         if self.output_file:
@@ -347,20 +358,32 @@ class DataTracker(CallbackTracker):
     
     @fill_in_docstring
     def __init__(self, func: Callable,
-                 interval: IntervalData = 1):
+                 interval: IntervalData = 1,
+                 filename: str = None):
         """ 
         Args:
-            func: The function to call periodically. The function signature
+            func:
+                The function to call periodically. The function signature
                 should be `(state)` or `(state, time)`, where `state` contains
                 the current state as an instance of
                 :class:`~pde.fields.FieldBase` and `time` is a
                 float value indicating the current time. Note that only a view
                 of the state is supplied, implying that a copy needs to be made
                 if the data should be stored.
+                Typical return values of the function are either a single
+                number, a numpy array, a list of number, or a dictionary to
+                return multiple numbers with assigned labels.
             interval:
                 {ARG_TRACKER_INTERVAL}
+            filename (str):
+                A path to a file to which the data is written at the end of the
+                tracking. The data format will be determined by the extension
+                of the filename. '.pickle' indicates a python pickle file
+                storing a tuple `(self.times, self.data)`, whereas any other
+                data format requires :mod:`pandas`.  
         """
         super().__init__(func=func, interval=interval)
+        self.filename = filename
         self.times: List[float] = []
         self.data: List[Any] = []
         
@@ -371,7 +394,8 @@ class DataTracker(CallbackTracker):
         Args:
             field (:class:`~pde.fields.FieldBase`):
                 The current state of the simulation
-            t (float): The associated time
+            t (float):
+                The associated time
         """
         self.times.append(t)
         if self._num_args == 1:
@@ -380,15 +404,64 @@ class DataTracker(CallbackTracker):
             self.data.append(self._callback(field, t))
         
         
+    def finalize(self, info: InfoDict = None) -> None:
+        """ finalize the tracker, supplying additional information
+
+        Args:
+            info (dict):
+                Extra information from the simulation        
+        """
+        super().finalize(info)
+        if self.filename:
+            self.to_file(self.filename)
+        
+        
     @property
-    def dataframe(self):
-        """ pandas.DataFrame: the data as a pandas DataFrame """
+    def dataframe(self) -> "pandas.DataFrame":
+        """ :class:`pandas.DataFrame`: the data in a dataframe
+        
+        If `func` returns a dictionary, the keys are used as column names.
+        Otherwise, the returned data is enumerated starting with '0'. In any
+        case the time point at which the data was recorded is stored in the
+        column 'time'.
+        """
         import pandas as pd
         df = pd.DataFrame(self.data)
         # insert the times and use them as an index
         df.insert(0, 'time', self.times)
         return df
+    
+    
+    def to_file(self, filename: str, **kwargs):
+        r""" store data in a file
         
+        The extension of the filename determines what format is being used. For
+        instance, '.pickle' indicates a python pickle file storing a tuple
+        `(self.times, self.data)`, whereas any other data format requires
+        :mod:`pandas`. Supported formats include 'csv', 'json'.  
+                
+        Args:
+            filename (str):
+                Path where the data is stored
+            \**kwargs:
+                Additional parameters may be supported for some formats 
+        """
+        extension = os.path.splitext(filename)[1].lower()
+        if extension == '.pickle':
+            # default 
+            import pickle
+            with open(filename, "wb") as fp:
+                pickle.dump((self.times, self.data), fp, **kwargs)
+            
+        elif extension == '.csv':
+            self.dataframe.to_csv(filename, **kwargs)
+        elif extension == '.json':
+            self.dataframe.to_json(filename, **kwargs)
+        elif extension in {'.xls', '.xlsx'}:
+            self.dataframe.to_excel(filename, **kwargs)
+        else:
+            raise ValueError(f'Unsupported file extension `{extension}`')
+            
             
             
 class SteadyStateTracker(TrackerBase):
@@ -425,7 +498,14 @@ class SteadyStateTracker(TrackerBase):
         
         
     def handle(self, field: FieldBase, t: float) -> None:
-        """ handle the data of `field` for a give `time` """
+        """ handle data supplied to this tracker
+        
+        Args:
+            field (:class:`~pde.fields.FieldBase`):
+                The current state of the simulation
+            t (float):
+                The associated time
+        """
         if self._last_data is not None:
             # scale with dt to make test independent of dt
             atol = self.atol * self.interval.dt
@@ -481,7 +561,14 @@ class RuntimeTracker(TrackerBase):
         
         
     def handle(self, field: FieldBase, t: float) -> None:
-        """ handle the data of `field` for a give `time` """
+        """ handle data supplied to this tracker
+        
+        Args:
+            field (:class:`~pde.fields.FieldBase`):
+                The current state of the simulation
+            t (float):
+                The associated time
+        """
         if time.time() > self.max_time:
             dt = timedelta(seconds=self.max_runtime)
             raise FinishedSimulation(f'Reached maximal runtime of {str(dt)}')
@@ -509,7 +596,14 @@ class ConsistencyTracker(TrackerBase):
         
         
     def handle(self, field: FieldBase, t: float) -> None:
-        """ handle the data of `field` for a give `time` """
+        """ handle data supplied to this tracker
+        
+        Args:
+            field (:class:`~pde.fields.FieldBase`):
+                The current state of the simulation
+            t (float):
+                The associated time
+        """
         if not np.all(np.isfinite(field.data)):
             raise StopIteration('Field was not finite')
             
@@ -561,7 +655,14 @@ class MaterialConservationTracker(TrackerBase):
         
         
     def handle(self, field: FieldBase, t: float) -> None:
-        """ handle the data of `field` for a give `time` """
+        """ handle data supplied to this tracker
+        
+        Args:
+            field (:class:`~pde.fields.FieldBase`):
+                The current state of the simulation
+            t (float):
+                The associated time
+        """
         if isinstance(field, FieldCollection):
             mags = np.array([f.magnitude for f in field])
         else:
